@@ -1,5 +1,6 @@
 import os
 import io
+import re
 import glob
 import urllib
 import zipfile
@@ -20,8 +21,17 @@ FILETYPES = {"tracks": [".flac", ".opus"],
              "images": [".jpeg", ".jpg", ".png"],
              "playlists": [".xspf"]}
 SORT_IMAGES = True
-STRICT_SLIDES = True  # True: show first booklet OR images OR default image
-                      # False: show merged booklets AND images
+STRICT_SLIDES = True    # True: show first booklet OR images OR default image
+                        # False: show merged booklets AND images
+ALT_ENCODINGS = True    # True: try alternative encodings of playlist filenames
+                        #       to match wrongly encoded filenames in ZIP
+                        # False: assume correct encoding of filenames in ZIP
+FIX_TRACKNUMBERS = True # True: skip leading zeros in tracknumber tag and
+                        #       assign succesive numbers if no tracknumber tag
+                        # False: leave leading zeros in tracknumbers and omit
+                        #        tracks without tracknumber tag
+FIX_DATE = True         # True: try "year" tag if no "date" tag is found
+                        # False: don't try "year" tag if no "date" tag is found
 
 def _get_content(files):
     content = {"tracks": [],
@@ -40,12 +50,11 @@ def _sort_images(images):
     back = []
     other = []
     for x in images:
-        if os.path.splitext(x)[0].endswith("folder") or \
-                os.path.splitext(x)[0].endswith("cover") or \
-                os.path.splitext(x)[0].endswith("front"):
-            front.append(x)
-        elif os.path.splitext(x)[0].endswith("back"):
+        parts = re.split(' |\_|\-|\.', os.path.splitext(x)[0])
+        if "back" in parts:
             back.append(x)
+        elif any(x in parts for x in ("front", "cover", "folder")):
+            front.append(x)
         else:
             other.append(x)
     return tuple(front + other + back)
@@ -56,26 +65,73 @@ def _create_booklet_page(args):
     from PIL import Image
     pdf = fitz.open(args[0])
     page = pdf[args[1]]
-    x, y = page.CropBox[2:]
+    try:
+        x, y = page.cropbox[2:]
+    except:
+        x, y = page.CropBox[2:]
     if x > y:
         factor = 2160 / x
     else:
         factor = 2160 / y
-    pix = page.getPixmap(matrix=fitz.Matrix(factor, factor))
-    pix.pillowWrite(args[2], format="JPEG", optimize=True)
+    try:
+        pix = page.get_pixmap(matrix=fitz.Matrix(factor, factor))
+        pix.pil_save(args[2], format="JPEG", optimize=True)
+    except:
+        pix = page.getPixmap(matrix=fitz.Matrix(factor, factor))
+        pix.pillowWrite(args[2], format="JPEG", optimize=True)
     pdf.close()
 
 
+def create_zipped_album(directory, filename=None):
+    """Create a Zipped Album from a directory.
+
+    Parameters
+    ----------
+    directory : str
+        the path to the directory to create the Zipped Album from
+    filename : str, optional
+        the path to the file to save the Zipped Album to
+        (if none is given, save as "<directory>.zlbm")
+
+    Returns
+    -------
+    filename : str
+        the path to the created Zipped Album file
+
+    """
+
+    if filename is None:
+        filename = directory + ".zlbm"
+
+    idx = 1
+    orig_filename = filename
+    while os.path.isfile(filename):
+        name, ext = os.path.splitext(orig_filename)
+        filename = f"{name} ({idx}){ext}"
+        idx += 1
+
+    assert os.path.isdir(directory)
+    content = _get_content(os.listdir(directory))
+    assert content["tracks"]
+    with zipfile.ZipFile(filename, 'w') as archive:
+        for file in (x for l in content.values() for x in l):
+            archive.write(os.path.join(directory, file), arcname=file)
+
+    return filename
+
+
 class ZippedAlbum:
     """A class representing a Zipped Album."""
 
-    def __init__(self, filename):
+    def __init__(self, filename, exact=False):
         """Create a ZippedAlbum object.
 
         Parameters
         ----------
         filename : str
             the path to the Zipped Album file.
+        exact : bool
+            no image sorting, no strict slides, no trying alternative encodings
 
         """
 
@@ -85,9 +141,20 @@ class ZippedAlbum:
         self._archive = zipfile.ZipFile(filename)
         self._tmpdir = tempfile.TemporaryDirectory()
         self._content = _get_content(sorted(self._archive.namelist()))
-
+        self._sort_images = SORT_IMAGES
+        self._strict_slides = STRICT_SLIDES
+        self._alt_encodings = ALT_ENCODINGS
+        self._fix_tracknumbers = FIX_TRACKNUMBERS
+        self._fix_date = FIX_DATE
         assert self._content["tracks"]
 
+        if exact:
+            self._sort_images = False
+            self._strict_slides = False
+            self._alt_encodings = False
+            self._fix_tracknumbers = False
+            self._fix_date = False
+
     def __del__(self):
         if hasattr(self, "_archive"):
             self._archive.close()
@@ -184,18 +251,19 @@ class ZippedAlbum:
                 self._artist = self.playlist["creator"]
             else:
                 try:
-                    artist = ["; ".join(v["tags"]["albumartist"]) \
-                              for k,v in self.tracks.items()]
+                    artists = ["; ".join(v["tags"]["albumartist"]) \
+                               for k,v in self.tracks.items()]
                 except:
-                    try:
-                        artist = ["; ".join(v["tags"]["artist"]) \
-                                  for k,v in self.tracks.items()]
-                    except:
-                        artist = ["Unknown Artist"]
-                if len(set(artist)) > 1:
+                    artists = []
+                    for k,v in self.tracks.items():
+                        try:
+                            artists.append("; ".join(v["tags"]["artist"]))
+                        except:
+                            artists.append("Unknown Artist")
+                if len(set(artists)) > 1:
                     self._artist = "Various Artists"
                 else:
-                    self._artist = artist[0]
+                    self._artist = artists[0]
             return self._artist
 
     @property
@@ -207,7 +275,15 @@ class ZippedAlbum:
                 self._year = dateutil.parser.parse(self.playlist["date"]).year
             else:
                 try:
-                    date = [v["tags"]["date"] for k,v in self.tracks.items()]
+                    date = []
+                    for k,v in self.tracks.items():
+                        try:
+                            date.append(v["tags"]["date"])
+                        except KeyError as e:
+                            if self._fix_date:
+                                date.append(v["tags"]["year"])
+                            else:
+                                raise e
                     lowest = 9999
                     highest = 0
                     for d in date:
@@ -235,31 +311,76 @@ class ZippedAlbum:
         if hasattr(self, "_tracklist"):
             return self._tracklist
         else:
+            artists = []
+            for k,v in self.tracks.items():
+                try:
+                    artists.append("; ".join(v["tags"]["artist"]))
+                except:
+                    artists.append("Unknown Artist")
             tracklist = []
             if "tracklist" in self.playlist:
-                #tracks = [self.tracks[x["location"]] \
-                          #for x in self.playlist["tracklist"]]
                 for c,x in enumerate(self.playlist["tracklist"]):
                     filename = x["location"]
-                    track = self.tracks[filename]
+                    try:
+                        track = self.tracks[filename]
+                    except:
+                        if self._alt_encodings:
+                            import pkgutil
+                            import encodings
+                            false_positives = set(["aliases"])
+                            encodings = set(
+                                name for imp, name, ispkg in \
+                                pkgutil.iter_modules(
+                                    encodings.__path__) if not ispkg)
+                            encodings.difference_update(false_positives)
+                            encodings = list(encodings)
+                            encodings.insert(0, encodings.pop(encodings.index(
+                                "latin_1")))
+                            encodings.insert(0, encodings.pop(encodings.index(
+                                "cp1252")))
+                            encodings.insert(0, encodings.pop(encodings.index(
+                                "utf_8")))
+                            continue_ = True
+                            for encoding in encodings: #("utf-8", "cp1252"):
+                                try:
+                                    filename = filename.encode(
+                                        encoding).decode("cp437")
+                                    track = self.tracks[filename]
+                                    continue_ = False
+                                    break
+                                except:
+                                    pass
+                            if continue_:
+                                continue
+                        else:
+                            continue
                     d = datetime.timedelta(
                         seconds=track["streaminfo"]["duration"])
                     d = str(d - datetime.timedelta(
                         microseconds=d.microseconds))
                     duration = d.lstrip("0:")
-                    try:
-                        if self.artist == "Various Artists":
+                    if len(set(artists)) > 1:
+                        try:
                             artist = "; ".join(track["tags"]["artist"])
+                        except:
+                            artist = "Unknown Artist"
+                        try:
                             title = "; ".join(track["tags"]["title"])
-                            name = f"{artist} - {title}"
-                        else:
+                        except:
+                            title = "Unknown Title"
+                        name = f"{artist} - {title}"
+                        if name == "Unknown Artist - Unknown Title":
+                            name = os.path.splitext(filename)[0]
+                    else:
+                        try:
                             name = "; ".join(track["tags"]["title"])
-                    except:
-                        name = filename
+                        except:
+                            name = os.path.splitext(filename)[0]
                     track["display"] = [str(c + 1), name, duration]
                     track["filename"] = filename
                     tracklist.append(track)
             else:
+                used_numbers = []
                 for nr, (filename, track) in enumerate(self.tracks.items()):
                     d = datetime.timedelta(
                         seconds=track["streaminfo"]["duration"])
@@ -269,23 +390,44 @@ class ZippedAlbum:
                     if int(d[0]) > 0:
                         duration = ":".join(d)
                     else:
-                        if int(d[1]) < 9:
+                        if int(d[1]) <= 9:
                             d[1] = d[1][1:]
                         duration = ":".join(d[1:])
-                    try:
-                        if self.artist == "Various Artists":
+                    if len(set(artists)) > 1:
+                        try:
                             artist = "; ".join(track["tags"]["artist"])
+                        except:
+                            artist = "Unknown Artist"
+                        try:
                             title = "; ".join(track["tags"]["title"])
-                            name = f"{artist} - {title}"
-                        else:
+                        except:
+                            title = "Unknown Title"
+                        name = f"{artist} - {title}"
+                        if name == "Unknown Artist - Unknown Title":
+                            name = os.path.splitext(filename)[0]
+                    else:
+                        try:
                             name = "; ".join(track["tags"]["title"])
-                    except:
-                        name = filename
+                        except:
+                            name = os.path.splitext(filename)[0]
                     try:
                         numbers = track["tags"]["tracknumber"]
+                        used_numbers.extend(numbers)
                     except:
-                        numbers = [str(nr)]
+                        if self._fix_tracknumbers:
+                            for x in range(nr + 1):
+                                if not str(x + 1) in used_numbers:
+                                    numbers = [str(x + 1)]
+                                    used_numbers.append(str(x + 1))
+                                    break
+                        else:
+                            continue
                     for number in numbers:
+                        if self._fix_tracknumbers:
+                            try:
+                                number = str(int(number))
+                            except:
+                                pass
                         track["display"] = [number, name, duration]
                         track["filename"] = filename
                         tracklist.append(track)
@@ -323,7 +465,7 @@ class ZippedAlbum:
             else:
                 booklet_pages = 0
             images = self._content["images"]
-            if STRICT_SLIDES:
+            if self._strict_slides:
                 if booklet_pages > 0:
                     self._nr_of_slides = booklet_pages
                 else:
@@ -347,7 +489,10 @@ class ZippedAlbum:
 
         """
 
-        return self._archive.open(self.tracklist[nr]["filename"])
+        try:
+            return self._archive.open(self.tracklist[nr]["filename"])
+        except:
+            pass
 
     def get_slide(self, nr):
         """Get a slide from the booklet.
@@ -373,7 +518,7 @@ class ZippedAlbum:
             return os.path.abspath(os.path.join(
             os.path.split(__file__)[0], "unknown_album.png"))
 
-        if STRICT_SLIDES:
+        if self._strict_slides:
             if self._content["booklets"]:
                 booklet_pages = self.nr_of_slides
             else:
@@ -388,7 +533,7 @@ class ZippedAlbum:
                 return None
         else:
             nr -= booklet_pages
-            if SORT_IMAGES:
+            if self._sort_images:
                 images = _sort_images(self._content["images"])
             else:
                 images = self._content["images"]
@@ -410,11 +555,15 @@ class ZippedAlbum:
                 booklet = fitz.open(
                     stream=self._archive.read(self._content["booklets"][0]),
                     filetype="pdf")
-                if not STRICT_SLIDES and len(self._content["booklets"]) > 1:
+                if not self._strict_slides and \
+                        len(self._content["booklets"]) > 1:
                     for other in self._content["booklets"][1:]:
                         doc = fitz.open(stream=self._archive.read(other),
                                         filetype="pdf")
-                        booklet.insertPDF(doc)
+                        try:
+                            booklet.insert_pdf(doc)
+                        except:
+                            booklet.insertPDF(doc)
                         doc.close()
                 path = os.path.join(self._tmpdir.name, "booklet.pdf")
                 booklet.save(path)
