@@ -33,7 +33,7 @@ except ModuleNotFoundError:
 from PIL import ImageTk, Image
 
 from .__meta__ import __author__, __version__
-from .album import ZippedAlbum
+from .album import ZippedAlbum, create_zipped_album
 from .binaries import has_ffmpeg, download_ffmpeg, get_platform
 
 
@@ -72,17 +72,19 @@ ABOUT_TEXT = """
 
  ┌──── Keyboard navigation ──────────────────────────────────────────────────┐
  │                                                                           │
- │  Play/Pause:                               Return           |      Space  │
- │  Select next track:                        Down             |      j      │
- │  Select previous track:                    Up               |      k      │
- │  Select first track:                       Home             |      gg     │
- │  Seek forward:                             Right            |      l      │
- │  Seek backward:                            Left             |      h      │
- │  Seek to beginning:                        Numpad 0         |      0      │
- │  Show next slide:                          Shift-Right      |      L      │
- │  Show previous slide:                      Shift-Left       |      H      │
- │  Increase volume:                          Shift-Right      |      K      │
- │  Decrease volume:                          Shift-Left       |      J      │
+ │  Play/Pause: ............................. Space ......... or ... Return  │
+ │  Select next track: ...................... Down .......... or ........ j  │
+ │  Select previous track: .................. Up ............ or ........ k  │
+ │  Select first track: ..................... Home .......... or ....... gg  │
+ │  Select last track: ...................... End ........... or ........ G  │
+ │  Seek forward: ........................... Right ......... or ........ l  │
+ │  Seek backward: .......................... Left .......... or ........ h  │
+ │  Seek to beginning: ...................... w ............. or ........ 0  │
+ │  Show next slide: ........................ Shift+Right ... or ........ L  │
+ │  Show previous slide: .................... Shift+Left .... or ........ H  │
+ │  Show first slide: ....................... W ............. or ........ )  │
+ │  Decrease volume: ........................ Shift+Down .... or ........ J  │
+ │  Increase volume: ........................ Shift+Up....... or ........ K  │
  │                                                                           │
  └───────────────────────────────────────────────────────────────────────────┘
 """.format(ver=__version__)
@@ -234,8 +236,15 @@ class TrackTooltip:
                           wraplength=self.treeview.winfo_width() - 120)
         label.grid(row=1, column= 0, padx=2, ipadx=0, ipady=0, sticky="nw")
         try:
-            year = "; ".join([str(
-                dateutil.parser.parse(x).year) for x in track["tags"]["date"]])
+            try:
+                year = "; ".join([str(dateutil.parser.parse(x).year) \
+                                  for x in track["tags"]["date"]])
+            except KeyError as e:
+                if self.album._fix_date:
+                    year = "; ".join([str(dateutil.parser.parse(x).year) \
+                                      for x in track["tags"]["year"]])
+                else:
+                    raise e
         except:
             year = "Unknown Year"
         label = ttk.Label(self.frame, text=year, justify=tk.LEFT,
@@ -344,7 +353,7 @@ class DownloadFFmpegDialogue:
             self.progressbar["value"] = percents
             self.text1["text"] = message
             self.text2["text"] = f"{percents} %"
-            self.top.update_idletasks()
+            self.top.update()
 
         try:
             download_ffmpeg(_progress)
@@ -388,6 +397,8 @@ class MainApplication(ttk.Frame):
         self.parent = parent
         self.style = ttk.Style()
         self.size = [WIDTH, HEIGHT]
+        self.repeat_album = tk.BooleanVar()
+        self.repeat_album.set(False)
         self.create_menu()
         self.create_widgets()
         self.remove_arrows()
@@ -396,23 +407,29 @@ class MainApplication(ttk.Frame):
         self.rowconfigure(0, weight=1)
         self.always_on_top = False
         self.fullscreen = False
-        self.repeat_album = False
         #self.create_bindings()
         self.loaded_album = None
         self.current_image = None
         self.selected_track_id = None
         self.playing_track_id = None
         self.resize_after_id = None
+        self.last_update_player = 0
+        self.now = time.monotonic
 
         def update_player():
             try:
                 if hasattr(self, "player") and self.player.is_playing:
                     self.player.update()
-                    track = self.loaded_album.tracklist[self.selected_track_id]
-                    time = self.player.time
-                    if str(self.playpause_button["state"]) == "normal":
-                        playhead = 100 / track["streaminfo"]["duration"] * time
-                        self.playhead = playhead
+                    now = self.now()
+                    if now - self.last_update_player >= 1:
+                        track = self.loaded_album.tracklist[
+                            self.selected_track_id]
+                        time = self.player.time
+                        if str(self.playpause_button["state"]) == "normal":
+                            playhead = 100 / track["streaminfo"]["duration"] \
+                                * time
+                            self.playhead = playhead
+                        self.last_update_player = now
                 self.after(UPDATE_INTERVALL, update_player)
             except tk._tkinter.TclError:
                 pass
@@ -453,15 +470,38 @@ class MainApplication(ttk.Frame):
                 sec_str = str(seconds).rjust(2, '0')
                 self.playhead_label["text"] = f"{min_str}:{sec_str}"
 
-    def _open_album(self, e=None):
+    def _open_album(self, *args, **kwargs):
+        if "exact" in kwargs:
+            exact = True
+        else:
+            exact = False
+        if self.loaded_album is not None:
+            initialdir = os.path.split(self.loaded_album.filename)[0]
+        else:
+            initialdir = os.getcwd()
         allowed_extensions = "*.zip *.zlbm"
         filetypes = [("Zipped Album files", allowed_extensions),
                      ("All files", "*.*")]
-        filename = filedialog.askopenfilename(filetypes=filetypes)
+        filename = filedialog.askopenfilename(initialdir=initialdir,
+                                              filetypes=filetypes)
         if filename:
-            self.load_album(filename)
+            self.load_album(filename, exact=exact)
             self.parent.focus_force()
 
+    def _create_album(self, *args, **kwargs):
+        if self.loaded_album is not None:
+            initialdir = os.path.split(self.loaded_album.filename)[0]
+        else:
+            initialdir = os.getcwd()
+        directory = filedialog.askdirectory(initialdir=initialdir)
+        if directory:
+            try:
+                self.make_album(directory)
+            except AssertionError:
+                print("Not a Zipped Album")
+            except:
+                print("Unknown error")
+
     def create_menu(self):
         self.menubar = tk.Menu(self.parent)
         if platform.system() == "Darwin":
@@ -479,14 +519,17 @@ class MainApplication(ttk.Frame):
         self.file_menu = tk.Menu(self.menubar, tearoff=False)
         self.menubar.add_cascade(menu=self.file_menu, label="File")
 
-        self.file_menu.add_command(label="Open",
+        self.file_menu.add_command(label="Open...",
                                    command=self._open_album,
                                    accelerator=f"{modifier}-O")
-
+        self.file_menu.add_command(label="Open exact...",
+                                   command=lambda: self._open_album(exact=True))
+        self.file_menu.add_command(label="Create...",
+                                   command=self._create_album)
         self.options_menu = tk.Menu(self.menubar, tearoff=False)
         self.menubar.add_cascade(menu=self.options_menu, label="Options")
         self.options_menu.add_checkbutton(label="Repeat",
-                                          command=self.toggle_repeat_album)
+                                          variable=self.repeat_album)
         self.view_menu = tk.Menu(self.menubar, tearoff=False)
         self.menubar.add_cascade(menu=self.view_menu, label=view_menu_label)
         self.view_presets_menu = tk.Menu(self.view_menu, tearoff=False)
@@ -638,6 +681,7 @@ class MainApplication(ttk.Frame):
         tree_frame.grid(column=0, row=1, sticky="nesw")
         tree_frame.columnconfigure(0, weight=1)
         tree_frame.rowconfigure(0, weight=1)
+        self.tree_frame = tree_frame
         self.style.configure('Treeview', relief="flat", borderwidth=1 * SCALING)
         self.tree = ttk.Treeview(tree_frame, show="tree", selectmode="browse")
         self.tree.tag_configure('normal', font=(FONTNAME, FONTSIZE))
@@ -675,6 +719,7 @@ class MainApplication(ttk.Frame):
 
         slider_frame = ttk.Frame(frame_bottom, height=int(10 * SCALING))
         slider_frame.grid(column=1, row=0, sticky="ew", padx=PADDING/2)
+        self.style.configure("TProgressbar", period = 0, maxphase =0)
         self.playhead_slider = ttk.Progressbar(slider_frame,
                                                orient="horizontal",
                                                mode='determinate', length=100,
@@ -694,7 +739,8 @@ class MainApplication(ttk.Frame):
         slider_frame.grid(column=2, row=0, sticky="ew", padx=PADDING/2)
         self.volume_slider = ttk.Progressbar(slider_frame,
                                              orient="horizontal",
-                                             mode='determinate', length=int(50 * SCALING),
+                                             mode='determinate',
+                                             length=int(50 * SCALING),
                                              maximum=100, value=100)
         self.volume_slider.place(relheight=1.0, relwidth=1.0)
         self.volume_label = ttk.Label(frame_bottom, anchor="center",
@@ -736,6 +782,7 @@ class MainApplication(ttk.Frame):
             self.parent.bind("<F1>", lambda e: HelpDialogue(self.master))
         self.parent.bind(f"<{modifier}-q>", lambda e: self.quit())
 
+        self._last_increment_track = self.now()
         def increment_track(step):
             selected_track_id = self.selected_track_id + step
             if 0 <= selected_track_id < len(self.loaded_album.tracklist):
@@ -743,10 +790,15 @@ class MainApplication(ttk.Frame):
                     return
                 play_next = False
                 if self.playing_track_id is not None:
+                    if self.now() - self._last_increment_track < 0.5:
+                        return
+                    self._last_increment_track = self.now()
                     self.pause()
                     play_next = True
                 selected_track_id = self.selected_track_id + step
                 self.tree.selection_set([str(selected_track_id)])
+                self.tree.focus(str(selected_track_id))
+                self.tree.see(str(selected_track_id))
                 self.selected_track_id = selected_track_id
                 self.load_track()
                 #self.parent.update()
@@ -764,8 +816,29 @@ class MainApplication(ttk.Frame):
             if self.selected_track_id not in (None, 0):
                 increment_track(-self.selected_track_id)
 
+        self._first_g_key_pressed = False
+        self._first_g_key_time = self.now()
+
+        def goto_first_track_vim(e):
+            if self.selected_track_id not in (None, 0):
+                if self._first_g_key_pressed:
+                    if self.now() - self._first_g_key_time < 1:
+                        increment_track(-self.selected_track_id)
+                        self._first_g_key_pressed = False
+                else:
+                    self._first_g_key_pressed = True
+                self._first_g_key_time = self.now()
+
         self.parent.bind("<Home>", goto_first_track)
-        self.parent.bind("<g><g>", goto_first_track)
+        self.parent.bind("<g>", goto_first_track_vim)
+
+        def goto_last_track(e):
+            length = len(self.loaded_album.tracklist) - 1
+            if self.selected_track_id not in (None, length):
+                increment_track(length - self.selected_track_id)
+
+        self.parent.bind("<End>", goto_last_track)
+        self.parent.bind("<G>", goto_last_track)
 
         def increment_playhead(step):
             if self.loaded_album is not None:
@@ -790,6 +863,7 @@ class MainApplication(ttk.Frame):
                 self.player.seek(0.0)
 
         self.parent.bind("0", seek_to_beginning)
+        self.parent.bind("<w>", seek_to_beginning)
 
         def increment_volume(step):
             self.volume += step
@@ -803,6 +877,8 @@ class MainApplication(ttk.Frame):
             if hasattr(self, "player"):
                 if str(self.playpause_button["state"]) == "disabled":
                     return
+                if self.now() - self._last_increment_track < 0.5:
+                        return
                 playing_id = self.playing_track_id
                 self.playpause()
                 if playing_id is not None:
@@ -823,6 +899,13 @@ class MainApplication(ttk.Frame):
         self.parent.bind(f"<Shift-Left>", lambda e: switch_image(-1))
         self.parent.bind(f"<H>", lambda e: switch_image(-1))
 
+        def switch_to_first_image(e):
+            if self.current_image not in (None, 0):
+                switch_image(-self.current_image)
+
+        self.parent.bind("<parenright>", switch_to_first_image)
+        self.parent.bind("<W>", switch_to_first_image)
+
         # Mouse (specific widgets)
         self.canvas.bind("<Enter>", lambda e: self.add_arrows())
         self.canvas.bind("<Leave>", lambda e: self.remove_arrows())
@@ -847,7 +930,8 @@ class MainApplication(ttk.Frame):
                 self.playhead = new_playhead
                 self.player.seek(pos)
 
-        self.playhead_slider.bind("<ButtonPress-1>", set_playhead_from_mouseclick)
+        self.playhead_slider.bind("<ButtonPress-1>",
+                                  set_playhead_from_mouseclick)
         self.playhead_slider.bind("<B1-Motion>", set_playhead_from_mouseclick)
 
         def set_volume_from_mouseclick(e):
@@ -871,6 +955,9 @@ class MainApplication(ttk.Frame):
                     self.pause()
                     play = True
                 selected_track_id = int(item_id)
+                self.tree.selection_set([str(selected_track_id)])
+                self.tree.focus(str(selected_track_id))
+                self.tree.see(str(selected_track_id))
                 self.selected_track_id = selected_track_id
                 self.load_track()
                 self.player.clear_on_queue = True
@@ -964,7 +1051,7 @@ class MainApplication(ttk.Frame):
         self.canvas_arrow_right = False
         self.canvas_arrow_left = False
 
-    def load_album(self, path):
+    def load_album(self, path, exact=False):
         # Clear current state
         self.parent.title("ZAP")
         self.hide_image()
@@ -978,13 +1065,19 @@ class MainApplication(ttk.Frame):
         self.title["text"] = ""
         self.artist["text"] = "Opening Album..."
         self.info["text"] = ""
+        self.tree_frame.grid_remove()
+        self.tree_frame.grid()
         self.parent.update()
 
         # Load new album
         try:
-            self.loaded_album = ZippedAlbum(path)
-        except AssertionError:
+            self.loaded_album = ZippedAlbum(path, exact=exact)
+        except:
             self.artist["text"] = "No Album"
+            file = os.path.split(path)[-1]
+            messagebox.showerror(
+                title="Error opening album",
+                message=f'"{file}" does not seem to be a valid Zipped Album!')
             return
 
         self.loaded_album.prepare_booklet_pages(self.wait_cover_image)
@@ -1019,7 +1112,7 @@ class MainApplication(ttk.Frame):
         else:
             width = WIDTH
             height = HEIGHT
-        c1_width = width - height -2 - c0_width - c2_width  # 2=2x1 frame borders
+        c1_width = width - height - 2 - c0_width - c2_width  # 2=2x1 frame borders
         self.tree.column('Title', width=c1_width)
         for c, track in enumerate(self.loaded_album.tracklist):
             if c % 2 == 1:
@@ -1029,6 +1122,8 @@ class MainApplication(ttk.Frame):
             self.tree.insert(parent='', index=c, iid=c, text='', tags=tags,
                              values=track["display"])
         self.tree.selection_set(["0"])
+        self.tree.focus("0")
+        self.tree.see("0")
         self.selected_track_id = 0
         self.truncate_titles()
 
@@ -1037,7 +1132,7 @@ class MainApplication(ttk.Frame):
         if len(set([type(x) for x in self.loaded_album.tracklist])) == 1:
             self.player = GaplessAudioPlayer()
         else:
-            self.player = AudioPlayer()
+            self.player = GaplessAudioPlayer()  #AudioPlayer()
 
         def next_gapless():
             if self.selected_track_id + 1 < len(self.loaded_album.tracklist):
@@ -1046,18 +1141,17 @@ class MainApplication(ttk.Frame):
                 dur = track["streaminfo"]["duration"]
                 tickspeed = UPDATE_INTERVALL / 1000
                 pos = self.playhead / 100 * dur + tickspeed
-                start = time.time()
+                start = self.now()
                 buffer_time = self.player.buffer_time
                 # Update GUI after running out of audio data
                 while True:
-                    current_time = time.time()
+                    current_time = self.now()
                     max_time = min(dur - (pos - tickspeed), 2 * buffer_time)
                     if current_time - start >= max_time:
                         break
-                    self.playhead = 100 / dur * (pos + time.time() - start)
-                    self.player.update(tick_only=True)
+                    self.playhead = 100 / dur * (pos + self.now() - start)
                     self.parent.update()
-                    sleep_time = tickspeed - (time.time() - current_time)
+                    sleep_time = tickspeed - (self.now() - current_time)
                     if sleep_time > 0:
                         time.sleep(sleep_time)
                 self.playpause_button["state"] = "normal"
@@ -1070,6 +1164,8 @@ class MainApplication(ttk.Frame):
                 tags = [x for x in tags if x != "normal"]
                 tags.append("bold")
                 self.tree.selection_set([str(track_id)])
+                self.tree.focus(str(track_id))
+                self.tree.see(str(track_id))
                 self.tree.item(str(track_id), tags=tags)
                 try:
                     self.player.queue(self.loaded_album.get_audio(track_id + 1))
@@ -1081,6 +1177,25 @@ class MainApplication(ttk.Frame):
                 self.set_title()
 
         def next():
+            self.playpause_button["state"] = "disabled"
+            track = self.loaded_album.tracklist[self.selected_track_id]
+            dur = track["streaminfo"]["duration"]
+            tickspeed = UPDATE_INTERVALL / 1000
+            pos = self.playhead / 100 * dur + tickspeed
+            start = self.now()
+            buffer_time = self.player.buffer_time
+            # Update playhead after running out of audio data
+            while True:
+                current_time = self.now()
+                max_time = min(dur - (pos - tickspeed), 2 * buffer_time)
+                if current_time - start >= max_time:
+                    break
+                self.playhead = 100 / dur * (pos + self.now() - start)
+                self.parent.update()
+                sleep_time = tickspeed - (self.now() - current_time)
+                if sleep_time > 0:
+                    time.sleep(sleep_time)
+            self.playpause_button["state"] = "normal"
             if self.selected_track_id + 1 < len(self.loaded_album.tracklist):
                 tags = self.tree.item(str(self.selected_track_id), "tags")
                 tags = [x for x in tags if x != "bold"]
@@ -1096,42 +1211,26 @@ class MainApplication(ttk.Frame):
                 tags = [x for x in tags if x != "normal"]
                 tags.append("bold")
                 self.tree.selection_set([str(track_id)])
+                self.tree.focus(str(track_id))
+                self.tree.see(str(track_id))
                 self.tree.item(str(track_id), tags=tags)
                 self.selected_track_id = track_id
                 self.playing_track_id = track_id
                 self.load_track()
                 self.set_title()
             else:
-                self.playpause_button["state"] = "disabled"
-                track = self.loaded_album.tracklist[self.selected_track_id]
-                dur = track["streaminfo"]["duration"]
-                tickspeed = UPDATE_INTERVALL / 1000
-                pos = self.playhead / 100 * dur + tickspeed
-                start = time.time()
-                buffer_time = self.player.buffer_time
-                # Update playhead after running out of audio data
-                while True:
-                    current_time = time.time()
-                    max_time = min(dur - (pos - tickspeed), 2 * buffer_time)
-                    if current_time - start >= max_time:
-                        break
-                    self.playhead = 100 / dur * (pos + time.time() - start)
-                    self.player.update(tick_only=True)
-                    self.parent.update()
-                    sleep_time = tickspeed - (time.time() - current_time)
-                    if sleep_time > 0:
-                        time.sleep(sleep_time)
-                self.playpause_button["state"] = "normal"
                 if self.playing_track_id is not None:
                     self.pause()
                 self.tree.selection_set(["0"])
+                self.tree.focus("0")
+                self.tree.see("0")
                 self.selected_track_id = 0
                 self.load_track()
                 self.set_title()
                 self.player.clear()
                 self.player.clear_on_queue = True
                 self.player.seek(0.0)
-                if self.repeat_album:
+                if self.repeat_album.get():
                     self.play()
 
         self.player.eos_callback = next
@@ -1144,6 +1243,37 @@ class MainApplication(ttk.Frame):
         self.set_title()
         self.create_menu()
 
+    def make_album(self, path, exact=False):
+        # Clear current state
+        self.parent.title("ZAP")
+        self.hide_image()
+        self.show_image(-1)
+        self.loaded_album = None
+        if self.playing_track_id is not None:
+            self.pause()
+        for i in self.tree.get_children():
+            self.tree.delete(i)
+        self.remove_arrows()
+        self.title["text"] = ""
+        self.artist["text"] = "Creating Album..."
+        self.info["text"] = ""
+        self.parent.update()
+
+        # Make new album
+        filename = None
+        try:
+            filename = create_zipped_album(path)
+        except:
+            self.artist["text"] = "No Album"
+            dir_ = os.path.split(path)[-1]
+            messagebox.showerror(
+                title="Error creating album",
+                message=f'"{dir_}" does not seem to be in a format to create '
+                'a Zipped Album from!')
+            return
+        if filename is not None:
+            self.load_album(filename, exact=exact)
+
     def truncate_titles(self, event=None):
         if self.loaded_album is None:
             return
@@ -1326,7 +1456,17 @@ class MainApplication(ttk.Frame):
             #    self.parent.wm_attributes("zoomed", True)
             #self.fit_to_slides()
             #return
-            max_width, max_height = self.parent.maxsize()
+            if platform.system() == "Windows":
+                self.parent.attributes("-alpha", 0)
+                self.parent.state('zoomed')
+                self.parent.update()
+                max_width  = self.parent.winfo_width()
+                max_height = self.parent.winfo_height()
+                self.parent.state('normal')
+                self.parent.update()
+                self.parent.attributes("-alpha", 1)
+            else:
+                max_width, max_height = self.parent.maxsize()
             if max_width > max_height:
                 width = max_height + (WIDTH - HEIGHT)
                 height = max_height
@@ -1381,12 +1521,6 @@ class MainApplication(ttk.Frame):
         self.show_image(current_image)
         self.truncate_titles()
 
-    def toggle_repeat_album(self):
-        if self.repeat_album:
-            self.repeat_album = False
-        else:
-            self.repeat_album = True
-
     def set_title(self):
         title = "ZAP"
         if self.loaded_album is not None:
@@ -1433,7 +1567,7 @@ def run():
                      tk.PhotoImage(file=os.path.abspath(os.path.join(
                 os.path.split(__file__)[0], "zipped_album_icon.png"))))
     app.pack(side="top", fill="both", expand=True)
-    root.geometry(f"{WIDTH}x{HEIGHT}")
+    root.geometry(f"{WIDTH}x{HEIGHT}+0+0")
     root.update()
     root.deiconify()
     root.minsize(WIDTH-HEIGHT, 0)
@@ -1466,7 +1600,26 @@ def run():
             sys.exit()
 
     try:
-        app.load_album(os.path.abspath(sys.argv[1]))
+        if "--exact" in sys.argv:
+            exact = True
+        else:
+            exact = False
+        if "--create" in sys.argv:
+            create = True
+        else:
+            create = False
+        if os.path.isdir(sys.argv[-1]):
+            if create:
+                app.make_album(os.path.abspath(sys.argv[-1]), exact=exact)
+            else:
+                os.chdir(os.path.abspath(sys.argv[-1]))
+        if os.path.splitext(sys.argv[-1])[-1] in (".zlbm", ".zip"):
+            if os.path.isfile(sys.argv[-1]):
+                app.load_album(os.path.abspath(sys.argv[-1]), exact=exact)
+            else:
+                messagebox.showerror(
+                title="Error opening album",
+                message=f'The file "{sys.argv[-1]}" does not exist!')
     except:
         pass
 
